@@ -307,71 +307,66 @@ You can control what lines will be unwrapped by frobbing
 indicating the minimum and maximum length of an unwrapped citation line.  If
 NODISPLAY is non-nil, don't redisplay the article buffer."
   (interactive "P")
-  (save-excursion
-    (let ((case-fold-search nil)
-	  (inhibit-read-only t)
-	  (cite-marks gnus-outlook-deuglify-cite-marks)
-	  (no-wrap gnus-outlook-deuglify-no-wrap-chars)
-	  (stop-chars gnus-outlook-deuglify-unwrap-stop-chars))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	(while (re-search-forward
-		(concat
-		 "^\\([ \t" cite-marks "]*\\)"
-		 "\\([" cite-marks "].*[^\n " stop-chars "]\\)[ \t]?\n"
-		 "\\1\\([^\n " cite-marks no-wrap "]+.*\\)$")
+  (let ((case-fold-search nil)
+	(inhibit-read-only t)
+	(cite-marks gnus-outlook-deuglify-cite-marks)
+	(no-wrap gnus-outlook-deuglify-no-wrap-chars)
+	(stop-chars gnus-outlook-deuglify-unwrap-stop-chars))
+    (gnus-with-article-buffer
+      (article-goto-body)
+      (while (re-search-forward
+	      (concat
+	       "^\\([ \t" cite-marks "]*\\)"
+	       "\\([" cite-marks "].*[^\n " stop-chars "]\\)[ \t]?\n"
+	       "\\1\\([^\n " cite-marks no-wrap "]+.*\\)$")
               nil t)
-	  (let ((len12 (- (match-end 2) (match-beginning 1)))
+	(let ((len12 (- (match-end 2) (match-beginning 1)))
 	      (len3 (- (match-end 3) (match-beginning 3))))
-	    (if (and (> len12 gnus-outlook-deuglify-unwrap-min)
+	  (when (and (> len12 gnus-outlook-deuglify-unwrap-min)
 		     (< (+ len12 len3) gnus-outlook-deuglify-unwrap-max))
-		(progn
-		  (replace-match "\\1\\2 \\3")
-		  (goto-char (match-beginning 0)))))))))
+	    (replace-match "\\1\\2 \\3")
+	    (goto-char (match-beginning 0)))))))
   (unless nodisplay (gnus-outlook-display-article-buffer)))
 
 (defun gnus-outlook-rearrange-article (attr-start)
   "Put the text from ATTR-START to the end of buffer at the top of the article buffer."
-  (save-excursion
-    (let ((inhibit-read-only t)
-	  (cite-marks gnus-outlook-deuglify-cite-marks))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	;; article does not start with attribution
-	(unless (= (point) attr-start)
-	  (gnus-kill-all-overlays)
-	  (let ((cur (point))
-		;; before signature or end of buffer
-		(to (if (gnus-article-search-signature)
-			(point)
-		      (point-max))))
-	    ;; handle the case where the full quote is below the
-	    ;; signature
-	    (if (< to attr-start)
-		(setq to (point-max)))
-	    (transpose-regions cur attr-start attr-start to)))))))
+  (let ((inhibit-read-only t)
+	(cite-marks gnus-outlook-deuglify-cite-marks))
+    (gnus-with-article-buffer
+      (article-goto-body)
+      ;; article does not start with attribution
+      (unless (= (point) attr-start)
+	(gnus-kill-all-overlays)
+	(let ((cur (point))
+	      ;; before signature or end of buffer
+	      (to (if (gnus-article-search-signature)
+		      (point)
+		    (point-max))))
+	  ;; handle the case where the full quote is below the
+	  ;; signature
+	  (if (< to attr-start)
+	      (setq to (point-max)))
+	  (transpose-regions cur attr-start attr-start to))))))
 
 ;; John Doe <john.doe@some.domain> wrote in message
 ;; news:a87usw8$dklsssa$2@some.news.server...
 
 (defun gnus-outlook-repair-attribution-outlook ()
   "Repair a broken attribution line (Outlook)."
-  (save-excursion
-    (let ((case-fold-search nil)
-	  (inhibit-read-only t)
-	  (cite-marks gnus-outlook-deuglify-cite-marks))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	(if (re-search-forward
+  (let ((case-fold-search nil)
+	(inhibit-read-only t)
+	(cite-marks gnus-outlook-deuglify-cite-marks))
+    (gnus-with-article-buffer
+      (article-goto-body)
+      (when (re-search-forward
 	     (concat "^\\([^" cite-marks "].+\\)"
 		     "\\(" gnus-outlook-deuglify-attrib-verb-regexp "\\)"
 		     "\\(.*\n?[^\n" cite-marks "].*\\)?"
 		     "\\(" gnus-outlook-deuglify-attrib-end-regexp "\\)$")
 	     nil t)
-	    (progn
-	      (gnus-kill-all-overlays)
-	      (replace-match "\\1\\2\\4")
-	      (match-beginning 0)))))))
+	(gnus-kill-all-overlays)
+	(replace-match "\\1\\2\\4")
+	(match-beginning 0)))))
 
 
 ;; ----- Original Message -----
@@ -382,42 +377,38 @@ NODISPLAY is non-nil, don't redisplay the article buffer."
 
 (defun gnus-outlook-repair-attribution-block ()
   "Repair a big broken attribution block."
-  (save-excursion
-    (let ((case-fold-search nil)
-	  (inhibit-read-only t)
-	  (cite-marks gnus-outlook-deuglify-cite-marks))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	(if (re-search-forward
-            (concat "^[" cite-marks " \t]*--* ?[^-]+ [^-]+ ?--*\\s *\n"
+  (let ((case-fold-search nil)
+	(inhibit-read-only t)
+	(cite-marks gnus-outlook-deuglify-cite-marks))
+    (gnus-with-article-buffer
+      (article-goto-body)
+      (when (re-search-forward
+	     (concat "^[" cite-marks " \t]*--* ?[^-]+ [^-]+ ?--*\\s *\n"
 		     "[^\n:]+:[ \t]*\\([^\n]+\\)\n"
 		     "\\([^\n:]+:[ \t]*[^\n]+\n\\)+")
 	     nil t)
-	    (progn
-	      (gnus-kill-all-overlays)
-	      (replace-match "\\1 wrote:\n")
-	      (match-beginning 0)))))))
+	(gnus-kill-all-overlays)
+	(replace-match "\\1 wrote:\n")
+	(match-beginning 0)))))
 
 ;; On Wed, 16 Jan 2002 23:23:30 +0100, John Doe <john.doe@some.domain> wrote:
 
 (defun gnus-outlook-repair-attribution-other ()
   "Repair a broken attribution line (other user agents than Outlook)."
-  (save-excursion
-    (let ((case-fold-search nil)
-	  (inhibit-read-only t)
-	  (cite-marks gnus-outlook-deuglify-cite-marks))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	(if (re-search-forward
+  (let ((case-fold-search nil)
+	(inhibit-read-only t)
+	(cite-marks gnus-outlook-deuglify-cite-marks))
+    (gnus-with-article-buffer
+      (article-goto-body)
+      (when (re-search-forward
 	     (concat "^\\("gnus-outlook-deuglify-attrib-cut-regexp"\\)?"
 		     "\\([^" cite-marks "].+\\)\n\\([^\n" cite-marks "].*\\)?"
 		     "\\(" gnus-outlook-deuglify-attrib-verb-regexp "\\).*"
 		     "\\(" gnus-outlook-deuglify-attrib-end-regexp "\\)$")
 	     nil t)
-	    (progn
-	      (gnus-kill-all-overlays)
-	      (replace-match "\\4 \\5\\6\\7")
-	      (match-beginning 0)))))))
+	(gnus-kill-all-overlays)
+	(replace-match "\\4 \\5\\6\\7")
+	(match-beginning 0)))))
 
 ;;;###autoload
 (defun gnus-article-outlook-repair-attribution (&optional nodisplay)
